//==========================================================================
// FoldableWindow.cpp
//   ܂Ȃ\EChE@\
//==========================================================================

#include "StdAfx.h"
#include "Util.h"
#include "MathUtil.h"
#include "FoldableWindow.h"
#include "WindowBacking.h"
#include "DirectX.h"



//-----------------------------------------------------------------------
// PFhԂ_
//-----------------------------------------------------------------------

WindowBackingRenderer_SingleColor::WindowBackingRenderer_SingleColor( HWND, COLORREF color )
	: brush( ::CreateSolidBrush(color) )
{
}

WindowBackingRenderer_SingleColor::~WindowBackingRenderer_SingleColor()
{
	::DeleteObject( brush );
}

void WindowBackingRenderer_SingleColor::on_paint( HWND wnd, HDC dc )
{
	RECT rc;
	::GetWindowRgnBox( wnd, &rc );
	::FillRect( dc, &rc, brush );
}



//-----------------------------------------------------------------------
// DirectXgĉ摜\_
//-----------------------------------------------------------------------

WindowBackingRenderer_Picture::WindowBackingRenderer_Picture( HWND wnd, const tstring& pictFile )
{
	direct3DDevice = DirectXManager::uniqueInstance().setUpDevice( wnd );
	texture        = DirectXManager::uniqueInstance().loadTexture( pictFile );
	if( direct3DDevice == NULL )
		alert( TEXT("DirectX s") );
}

WindowBackingRenderer_Picture::~WindowBackingRenderer_Picture()
{
	// direct3DDeviceDirectXManagerLĂ̂ReleaseȂ
	if( texture )
		texture->Release();
}

void WindowBackingRenderer_Picture::renderBegin()
{
	direct3DDevice->BeginScene();
	direct3DDevice->SetVertexShader( D3DFVF_CUSTOMVERTEX );
	direct3DDevice->SetTexture( 0, texture );
}

void WindowBackingRenderer_Picture::renderOneWindow( const FoldedRect& fr )
{
	vector<CUSTOMVERTEX> verts;
	fr.current_ura_shape( verts );
	direct3DDevice->DrawPrimitiveUP(
		D3DPT_TRIANGLEFAN, verts.size()-2, &verts[0], sizeof(CUSTOMVERTEX) );
}

void WindowBackingRenderer_Picture::renderEnd()
{
	direct3DDevice->EndScene();
}

void WindowBackingRenderer_Picture::on_display_size_change( HWND wnd )
{
	direct3DDevice = DirectXManager::uniqueInstance().setUpDevice( wnd );
}

void WindowBackingRenderer_Picture::on_paint( HWND wnd, HDC dc )
{
	if( D3DERR_DEVICELOST == direct3DDevice->Present(NULL,NULL,NULL,NULL) )
	{
		direct3DDevice = DirectXManager::uniqueInstance().setUpDevice( wnd );
		direct3DDevice->Present(NULL,NULL,NULL,NULL);
	}
}



//-----------------------------------------------------------------------
// ̗n
//-----------------------------------------------------------------------

WindowBacking::WindowBacking()
	: wnd( ::CreateWindowEx(
			WS_EX_TOOLWINDOW|WS_EX_TOPMOST|WS_EX_NOACTIVATE,
			getWindowClass(), TEXT("OriMado_UraMado"), WS_POPUP|WS_DISABLED,
			0, 0, ::GetSystemMetrics(SM_CXSCREEN), ::GetSystemMetrics(SM_CYSCREEN),
			NULL, NULL, current_module(), this ) )
{
}

WindowBacking::~WindowBacking()
{
	::DestroyWindow( wnd );
}

const TCHAR* WindowBacking::getWindowClass()
{
	static WNDCLASSEX wc;
	if( wc.lpszClassName == NULL )
	{
		wc.cbSize        = sizeof(wc);
		wc.lpszClassName = TEXT("OriMado_UraMado");
		wc.hInstance     = current_module();
		wc.hbrBackground = reinterpret_cast<HBRUSH>(1+COLOR_ACTIVECAPTION);
		wc.lpfnWndProc   = &WndProc;
		::RegisterClassEx( &wc );
	}
	return wc.lpszClassName;
}

void WindowBacking::dragBegin()
{
	first_render = true;
}

void WindowBacking::dragEnd()
{
	::ShowWindow( wnd, SW_HIDE );
}

LRESULT CALLBACK WindowBacking::WndProc( HWND wnd, UINT msg, WPARAM wp, LPARAM lp )
{
	if( msg == WM_CREATE )
	{
		// USERDATA  this |C^Ƃ
		::SetWindowLongPtr(
			wnd, GWLP_USERDATA,
		    reinterpret_cast<LONG_PTR>(reinterpret_cast<CREATESTRUCT*>(lp)->lpCreateParams) );
	}
	else if( msg == WM_DISPLAYCHANGE )
	{
		// this |C^o
		WindowBacking* pThis =
			reinterpret_cast<WindowBacking*>( ::GetWindowLongPtr( wnd, GWLP_USERDATA ) );

		// EChETCYύX
		int w = ::GetSystemMetrics(SM_CXSCREEN), h = ::GetSystemMetrics(SM_CYSCREEN);
		::SetWindowPos( wnd, HWND_TOP, 0, 0, w, h, 0 );

		// ʒm
		if( pThis->renderer )
			pThis->renderer->on_display_size_change( wnd );
	}
	else if( msg == WM_ERASEBKGND )
	{
		// this |C^o
		WindowBacking* pThis =
			reinterpret_cast<WindowBacking*>( ::GetWindowLongPtr( wnd, GWLP_USERDATA ) );

		// ʒm
		if( pThis->renderer )
			if( pThis->renderer->on_erase_background(wnd,reinterpret_cast<HDC>(wp)) )
				return TRUE;
	}
	else if( msg == WM_PAINT )
	{
		// this |C^o
		WindowBacking* pThis =
			reinterpret_cast<WindowBacking*>( ::GetWindowLongPtr( wnd, GWLP_USERDATA ) );

		// ʒm
		PAINTSTRUCT ps;
		::BeginPaint( wnd, &ps );
		if( pThis->renderer )
			pThis->renderer->on_paint(wnd,ps.hdc);
		::EndPaint( wnd, &ps );
		return TRUE;
	}
	return ::DefWindowProc( wnd, msg, wp, lp );
}

void WindowBacking::renderBegin()
{
	rgn_holder   = ::CreateRectRgn(0,0,0,0);
	render_count = 0;
	if( renderer )
		renderer->renderBegin();
}

void WindowBacking::renderOneWindow( const FoldedRect& shape, HRGN original_rgn )
{
	if( shape.is_folded() )
	{
		HRGN rgn = shape.current_ura_cut( original_rgn );
		::CombineRgn( rgn_holder, rgn_holder, rgn, RGN_OR );
		::DeleteObject(rgn);
		++ render_count;

		if( renderer )
			renderer->renderOneWindow( shape );
	}
}

void WindowBacking::renderEnd()
{
	if( renderer )
		renderer->renderEnd();
	::SetWindowRgn( wnd, rgn_holder, TRUE );
	if( render_count >= 1 )
	{
		if( first_render )
		{
			::SetWindowPos( wnd, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOSIZE|SWP_NOMOVE|SWP_SHOWWINDOW );
			first_render = false;
		}
		::InvalidateRect( wnd, NULL, TRUE );
		::UpdateWindow( wnd );
	}
}
